unit LUTMakerForm;

{
Revision history:

V1.0.0  2000 Jul 02  First released version
V1.0.2  2000 Jul 03  Add 32 x 32 icon
V1.0.4  2001 Jan 27  Make tab order more logical
                     Recompile for Delphi 5 runtime library
V1.0.6  2001 Dec 12  Add Save as (also used when file exists)

Copyright  David J Taylor, Edinburgh
Web site:  www.satsignal.net
E-mail:    davidtaylor@writeme.com
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, RegistryVariables, ComCtrls, Menus;

type
  TFormLUTmaker = class(TForm)
    ImageLUT: TImage;
    LabelM60: TLabel;
    LabelM50: TLabel;
    LabelM40: TLabel;
    LabelM30: TLabel;
    LabelM20: TLabel;
    LabelM10: TLabel;
    LabelO0: TLabel;
    LabelP10: TLabel;
    LabelP20: TLabel;
    LabelP30: TLabel;
    LabelP40: TLabel;
    ButtonClose: TButton;
    ButtonSave: TButton;
    PaintBoxM60: TPaintBox;
    PaintBoxM50: TPaintBox;
    PaintBoxM40: TPaintBox;
    PaintBoxM30: TPaintBox;
    PaintBoxM20: TPaintBox;
    PaintBoxM10: TPaintBox;
    PaintBoxO0: TPaintBox;
    PaintBoxP10: TPaintBox;
    PaintBoxP20: TPaintBox;
    PaintBoxP30: TPaintBox;
    PaintBoxP40: TPaintBox;
    ColorDialog: TColorDialog;
    CheckBoxSmooth: TCheckBox;
    GroupBoxColourEditor: TGroupBox;
    EditHue: TEdit;
    UpDownHue: TUpDown;
    LabelHue: TLabel;
    EditSat: TEdit;
    UpDownSat: TUpDown;
    LabelSat: TLabel;
    ButtonUpdateColour: TButton;
    LabelTempIndex: TLabel;
    PaintBoxTest: TPaintBox;
    PaintBoxTry: TPaintBox;
    LabelOld: TLabel;
    LabelNew: TLabel;
    ButtonFactoryReset: TButton;
    CheckBoxWhiteTop: TCheckBox;
    EditTailPercent: TEdit;
    UpDownTailPercent: TUpDown;
    LabelBlueTail: TLabel;
    LabelCelsius: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileExit: TMenuItem;
    N1: TMenuItem;
    Save1: TMenuItem;
    Help1: TMenuItem;
    HelpAbout: TMenuItem;
    LabelBitmap: TLabel;
    GroupBoxMonitorCheck: TGroupBox;
    PaintBoxBlack: TPaintBox;
    PaintBoxWhite: TPaintBox;
    PaintBoxM70: TPaintBox;
    SaveDialog: TSaveDialog;
    FileSaveAs: TMenuItem;
    procedure FormCreate (Sender: TObject);
    procedure FormDestroy (Sender: TObject);
    procedure FormResize (Sender: TObject);
    procedure ButtonCloseClick (Sender: TObject);
    procedure PaintBoxSamplePaint (Sender: TObject);
    procedure PaintBoxSampleClick (Sender: TObject);
    procedure RadioGroupColourModelClick (Sender: TObject);
    procedure CheckBoxSmoothClick (Sender: TObject);
    procedure PaintBoxTestPaint (Sender: TObject);
    procedure PaintBoxTryPaint (Sender: TObject);
    procedure ColourChangeClick (Sender: TObject; Button: TUDBtnType);
    procedure ButtonUpdateColourClick (Sender: TObject);
    procedure CheckBoxBlueTailClick (Sender: TObject);
    procedure ButtonFactoryResetClick (Sender: TObject);
    procedure ButtonSaveClick (Sender: TObject);
    procedure CheckBoxWhiteTopClick (Sender: TObject);
    procedure UpDownTailPercentClick (Sender: TObject; Button: TUDBtnType);
    procedure HelpAboutClick (Sender: TObject);
    procedure FormCloseQuery (Sender: TObject; var CanClose: Boolean);
    procedure FileExitClick (Sender: TObject);
    procedure PaintBoxMonitorCheckPaint (Sender: TObject);
    procedure EditHueChange (Sender: TObject);
    procedure EditSatChange (Sender: TObject);
    procedure FileSaveAsClick(Sender: TObject);
  private
    { Private declarations }
    LUTbitmap: TBitmap;
    hues: array [-7..4] of TRegistryInteger;
    sats: array [-7..4] of TRegistryInteger;
    colours: array [-7..+4] of TColor;
    white_top: TRegistryBoolean;
    smooth_colours: TRegistryBoolean;
    blue_tail_percent: TRegistryInteger;
    new_colour: TColor;
    new_hue: integer;
    new_sat: integer;
    SatSignal_path: string;
    procedure EnableColourEditor (const enable: boolean);
    procedure FactoryReset;
    function IndexToStr (const i: integer): string;
    procedure UpdateColours;
    procedure UpdateTestPatch;
  public
    { Public declarations }
  end;

var
  FormLUTmaker: TFormLUTmaker;

implementation

{$R *.DFM}

uses
  Thermal, AboutLUT;


procedure TFormLUTmaker.FormCreate (Sender: TObject);
var
  t: integer;
  r, g, b: single;
begin
  RegisterThisApp;
  LUTbitmap := TBitmap.Create;
  LUTbitmap.PixelFormat := pf24bit;
  LUTbitmap.Width := 100;
  LUTbitmap.Height := 256;
  ImageLUT.Picture.Bitmap := LUTbitmap;
  hues [-7] := TRegistryInteger.CreateValue ('LUThue' + IndexToStr (-7), -1);
  sats [-7] := TRegistryInteger.CreateValue ('LUTsat' + IndexToStr (-7), -1);
  for t := -6 to 4 do
    begin
    hues [t] := TRegistryInteger.CreateValue ('LUThue' + IndexToStr (t), 0);
    sats [t] := TRegistryInteger.CreateValue ('LUTsat' + IndexToStr (t), 0);
    end;
  if hues [-7].Value = -1 then FactoryReset;
  for t := -7 to 4 do
    begin
    HSVtoRGB (hues [t].Value, sats [t].Value / 100, 1.0, r, g, b);
    colours [t] := RGB (Round (r * 255), Round (g * 255), Round (b * 255));
    end;
  blue_tail_percent := TRegistryInteger.CreateValue ('LUTTailPercent', 12);
  UpDownTailPercent.Position := blue_tail_percent.Value;
  white_top := TRegistryBoolean.CreateValue ('LUTWhiteTop', True);
  CheckBoxWhiteTop.Checked := white_top.Value;
  smooth_colours := TRegistryBoolean.CreateValue ('LUTSmoothColours', True);
  CheckBoxSmooth.Checked := smooth_colours.Value;
  EnableColourEditor (False);
  DoubleBuffered := True;
  SatSignal_path := ExtractFilePath (GetRegisteredAppPath ('SatSignal'));
end;


procedure TFormLUTmaker.FormDestroy (Sender: TObject);
var
  t: integer;
begin
  LUTbitmap.Free;
  smooth_colours.Value := CheckBoxSmooth.Checked;
  smooth_colours.Free;
  blue_tail_percent.Value := UpDownTailPercent.Position;
  blue_tail_percent.Free;
  white_top.Value := CheckBoxWhiteTop.Checked;
  white_top.Free;
  for t := -7 to 4 do
    begin
    hues [t].Free;
    sats [t].Free;
    end;
end;


procedure TFormLUTmaker.FormCloseQuery (Sender: TObject;
  var CanClose: Boolean);
begin
  ImageLUT.Picture.Bitmap := nil;
  LUTbitmap.Free;
  LUTbitmap := nil;
end;


procedure TFormLUTmaker.FileExitClick (Sender: TObject);
begin
  Close;
end;


procedure TFormLUTmaker.ButtonCloseClick (Sender: TObject);
begin
  Close;
end;


function TFormLUTmaker.IndexToStr (const i: integer): string;
var
  name: string;
begin
  name := Trim (IntToStr (10 * i));
  if i < 0
    then Result := 'M' + Copy (name, 2, 9)
    else
      if i > 0
        then Result := 'P' + name
        else Result := 'O' + name;
end;


procedure TFormLUTmaker.FormResize (Sender: TObject);
var
  t: integer;
  l: TLabel;
  p: TPaintBox;
  x, y: integer;
begin
  for t := -7 to 4 do
    begin
    l := TLabel (FindComponent ('Label' + IndexToStr (t)));
    x := ImageLUT.Left + ImageLUT.Width * (t + 6) div 10;
    y := ImageLUT.Top + ImageLUT.Height + 3;
    if l <> nil then
      begin
      l.Left := x - l.Width div 2;
      l.Top := y;
      end;
    LabelCelsius.Top := y;
    p := TPaintBox (FindComponent ('PaintBox' + IndexToStr (t)));
    if p <> nil then
      begin
      if t = -7
      then
        begin
        p.Left := EditTailPercent.Left + EditTailPercent.Width +
                  UpDownTailPercent.Width + 8;
        p.Top := EditTailPercent.Top + 3;
        end
      else
        begin
        p.Left := x - p.Width div 2;
        p.Top := y + 18;
        end;
      end;
    end;
  UpdateColours;
end;


procedure TFormLUTmaker.PaintBoxSamplePaint (Sender: TObject);
var
  pbx: TPaintBox;
  cnv: TCanvas;
begin
  if Sender is TPaintBox then
    begin
    pbx := Sender as TPaintBox;
    cnv := pbx.Canvas;
    with cnv do
      begin
      Brush.Color := colours [pbx.Tag];
      FillRect (Rect (0, 0, pbx.Width, pbx.Height));
      end;
    end;
end;


procedure TFormLUTmaker.PaintBoxTestPaint (Sender: TObject);
begin
  if (PaintBoxTest.Tag >= -7) and (PaintBoxTest.Tag <= 4) then
    PaintBoxSamplePaint (Sender);
end;


procedure TFormLUTmaker.PaintBoxTryPaint (Sender: TObject);
begin
  if (PaintBoxTest.Tag >= -7) and (PaintBoxTest.Tag <= 4) then
    with PaintBoxTry, PaintBoxTry.Canvas do
      begin
      Brush.Color := new_colour;
      FillRect (Rect (0, 0, Width, Height));
      end;
end;


procedure TFormLUTmaker.EnableColourEditor (const enable: boolean);
begin
  GroupBoxColourEditor.Enabled := enable;
  LabelTempIndex.Enabled := enable;
  ButtonUpdateColour.Enabled := enable;
  LabelHue.Enabled := enable;
  LabelSat.Enabled := enable;
  EditHue.Enabled := enable;
  EditSat.Enabled := enable;
  LabelOld.Enabled := enable;
  LabelNew.Enabled := enable;
end;


procedure TFormLUTmaker.PaintBoxSampleClick (Sender: TObject);
var
  pbx: TPaintBox;
  clr: integer;
begin
  if Sender is TPaintBox then
    begin
    pbx := Sender as TPaintBox;
    clr := pbx.Tag;
    UpDownHue.Position := hues [clr].Value;
    UpDownSat.Position := sats [clr].Value;
    PaintBoxTest.Tag := clr;
    PaintBoxTest.Invalidate;
    if clr = -7
      then LabelTempIndex.Caption := 'blue tail'
      else LabelTempIndex.Caption := IntToStr (10 * clr) + ' C';
    EnableColourEditor (True);
    PaintBoxTry.Invalidate;
    end;
end;


procedure TFormLUTmaker.UpdateColours;
var
  bright: integer;
  temp: integer;
  t1, t2: integer;
  frac: single;
  r1, r2, g1, g2, b1, b2: single;
  r, g, b: single;
  h, s, v: single;
  sat: single;
  white_top: boolean;
  tail_percent: integer;
  pbx: TPaintBox;
begin
  if LUTbitmap = nil then Exit;
  
  tail_percent := UpDownTailPercent.Position;
  white_top := CheckBoxWhiteTop.Checked;
  ImageLUT.Picture.Bitmap := nil;
  for temp := -60 to 39 do
    begin
    t1 := ((temp + 60) div 10) - 6;
    t2 := t1 + 1;

    if (temp > -2) and (temp < +2) then
      begin
      t1 := 0;
      t2 := 0;
      end;

    r1 := (colours [t1] and $0000FF) / 255.0;
    g1 := ((colours [t1] and $00FF00) shr 8) / 255.0;
    b1 := ((colours [t1] and $FF0000) shr 16) / 255.0;
    r2 := (colours [t2] and $0000FF) / 255.0;
    g2 := ((colours [t2] and $00FF00) shr 8) / 255.0;
    b2 := ((colours [t2] and $FF0000) shr 16) / 255.0;

    frac := ((temp + 100 )mod 10) / 10;
    if CheckBoxSmooth.Checked
    then
      begin
      r := (1 - frac) * r1 + frac * r2;
      g := (1 - frac) * g1 + frac * g2;
      b := (1 - frac) * b1 + frac * b2;
      end
    else
      if temp > 0
      then
        begin
        r := r2;
        g := g2;
        b := b2;
        end
      else
        begin
        r := r1;
        g := g1;
        b := b1;
        end;

    RGBtoHSV (r, g, b, h, s, v);

    if white_top
      then sat := s * 1.5
      else sat := s;
    if sat > 1.0 then sat := 1.0;

    for bright := 0 to 255 do
      begin
      if white_top
      then
        begin
        if bright > 127
          then HSVtoRGB (h, sat * (255 - bright) / 127, bright / 255.0, r, g, b)
          else HSVtoRGB (h, sat, bright / 255.0, r, g, b);
        end
      else
        HSVtoRGB (h, sat, bright / 255.0, r, g, b);
      if (temp > 1) and ((bright *  100 div 255) < tail_percent) then
        begin
        v := 0.35 * (1 + temp / 39);
        HSVtoRGB (hues [-7].Value, sats [-7].Value / 100, v, r, g, b);
        end;
      r := 255.9 * r;
      g := 255.9 * g;
      b := 255.9 * b;
      LUTbitmap.Canvas.Pixels [temp + 60, 255 - bright] := RGB (Trunc (r), Trunc (g), Trunc (b));
      end;
    end;

  LUTbitmap.Modified := True;
  ImageLUT.Picture.Bitmap := LUTbitmap;

  for temp := -7 to 4 do
    begin
    pbx := TPaintBox (FindComponent ('PaintBox' + IndexToStr (temp)));
    if pbx <> nil then pbx.Repaint;
    end;
end;


procedure TFormLUTmaker.RadioGroupColourModelClick (Sender: TObject);
begin
  UpdateColours;
end;


procedure TFormLUTmaker.CheckBoxSmoothClick (Sender: TObject);
begin
  UpdateColours;
end;


procedure TFormLUTmaker.UpdateTestPatch;
var
  r, g, b: single;
begin
  new_hue := UpDownHue.Position;
  new_sat := UpDownSat.Position;
  HSVtoRGB (new_hue, new_sat / 100, 1.0, r, g, b);
  new_colour := RGB (Round (r * 255), Round (g * 255), Round (b * 255));
  PaintBoxTry.Invalidate;
end;


procedure TFormLUTmaker.EditSatChange (Sender: TObject);
begin
  try
    UpDownSat.Position := StrToInt (EditSat.Text);
    UpdateTestPatch;
  except
  end;
end;


procedure TFormLUTmaker.EditHueChange (Sender: TObject);
begin
  try
    UpDownHue.Position := StrToInt (EditHue.Text);
    UpdateTestPatch;
  except
  end;
end;


procedure TFormLUTmaker.ColourChangeClick (Sender: TObject;  Button: TUDBtnType);
begin
  UpdateTestPatch;
end;


procedure TFormLUTmaker.ButtonUpdateColourClick (Sender: TObject);
var
  clr_index: integer;
begin
  clr_index := PaintBoxTest.Tag;
  if (clr_index >= -7) and (clr_index <= 4) then
    begin
    colours [clr_index] := new_colour;
    hues [clr_index].Value := new_hue;
    sats [clr_index].Value := new_sat;
    UpdateColours;
    end;
end;


procedure TFormLUTmaker.CheckBoxBlueTailClick (Sender: TObject);
begin
  UpdateColours;
end;


procedure TFormLUTmaker.CheckBoxWhiteTopClick (Sender: TObject);
begin
  UpdateColours;
end;


procedure TFormLUTmaker.FactoryReset;
var
  t: integer;
  r, g, b: single;
begin
  // This is the blue tail value
  hues [-7].Value := 220;
  sats [-7].Value :=  70;

  hues [-6].Value := 310;
  sats [-6].Value :=   7;
  hues [-5].Value := 270;
  sats [-5].Value :=  12;
  hues [-4].Value := 250;
  sats [-4].Value :=  14;
  hues [-3].Value := 225;
  sats [-3].Value :=  14;
  hues [-2].Value := 205;
  sats [-2].Value :=  12;
  hues [-1].Value := 185;
  sats [-1].Value :=   9;
  hues [0].Value  := 100;
  sats [0].Value  :=   0;
  hues [1].Value  :=  85;
  sats [1].Value  :=  12;
  hues [2].Value  :=  55;
  sats [2].Value  :=  19;
  hues [3].Value  :=  35;
  sats [3].Value  :=  25;
  hues [4].Value  :=  15;
  sats [4].Value  :=  25;
  
  for t := -7 to 4 do
    begin
    HSVtoRGB (hues [t].Value, sats [t].Value / 100, 1.0, r, g, b);
    colours [t] := RGB (Round (r * 255), Round (g * 255), Round (b * 255));
    end;
end;


procedure TFormLUTmaker.ButtonFactoryResetClick (Sender: TObject);
begin
  FactoryReset;
  UpdateColours;
end;


procedure TFormLUTmaker.ButtonSaveClick (Sender: TObject);
var
  lut_filename: string;
begin
  lut_filename := SatSignal_path + 'TempToColourUser.bmp';
  if FileExists (lut_filename)
  then
    FileSaveAs.Click
  else
    begin
    LUTbitmap.SaveToFile (lut_filename);
    ShowMessage ('Bitmap saved to ' + ExpandFileName (lut_filename));
    end;
end;


procedure TFormLUTmaker.UpDownTailPercentClick (Sender: TObject;
  Button: TUDBtnType);
begin
  UpdateColours;
end;


procedure TFormLUTmaker.HelpAboutClick (Sender: TObject);
begin
  AboutBoxDJT1.ShowModal;
end;


procedure TFormLUTmaker.PaintBoxMonitorCheckPaint (Sender: TObject);
const
  clDarkGrey: integer = $00202020;
  clLightGrey: integer = $00F0F0F0;
var
  pbx: TPaintBox;
  outer_colour, inner_colour: TColor;
begin
  outer_colour := clBlack;
  inner_colour := clDarkGrey;
  pbx := nil;
  if Sender = PaintBoxBlack then pbx := PaintBoxBlack;
  if Sender = PaintBoxWhite then
    begin
    pbx := PaintBoxWhite;
    inner_colour := clLightGrey;
    outer_colour := clWhite;
    end;
  if pbx <> nil then
    with pbx, pbx.Canvas do
      begin
      Brush.Color := outer_colour;
      Pen.Color := outer_colour;
      Ellipse (0, 0, Width, Height);
      Brush.Color := inner_colour;
      Pen.Color := inner_colour;
      Ellipse (Width div 4, Height div 4, 3 * Width div 4, 3 * Height div 4);
      end;
end;

procedure TFormLUTmaker.FileSaveAsClick (Sender: TObject);
var
  lut_filename: string;
begin
  SaveDialog.InitialDir := SatSignal_path;
  if SaveDialog.Execute then
    begin
    try
      lut_filename := SaveDialog.FileName;
      LUTbitmap.SaveToFile (lut_filename);
      ShowMessage ('Bitmap saved to ' + ExpandFileName (lut_filename));
    except
    end;
    end;
end;

end.

